@@ -1,5 +1,5 @@
 package Yancy::Controller::Yancy::API;
-our $VERSION = '1.022';
+our $VERSION = '1.023';
 # ABSTRACT: An OpenAPI REST controller for the Yancy editor
 
 #pod =head1 DESCRIPTION
@@ -48,6 +48,10 @@ use Mojo::Base 'Mojolicious::Controller';
 #pod List the items in a collection. The collection name should be in the
 #pod stash key C<collection>.
 #pod
+#pod Each returned item will be filtered by filters conforming with
+#pod L<Mojolicious::Plugin::Yancy/yancy.filter.add> that are passed in the
+#pod array-ref in stash key C<filters_out>.
+#pod
 #pod C<$limit>, C<$offset>, and C<$order_by> may be provided as query parameters.
 #pod
 #pod =cut
@@ -78,8 +82,12 @@ sub list_items {
         $filter{ $key } = { -like => $value };
     }
 
-    my $res = $c->yancy->backend->list( $c->stash( 'collection' ), \%filter, \%opt );
+    my $coll = $c->stash( 'collection' );
+    my $res = $c->yancy->backend->list( $coll, \%filter, \%opt );
     _delete_null_values( @{ $res->{items} } );
+    $res->{items} = [
+        map _apply_op_filters( $coll, $_, $c->stash( 'filters_out' ), $c->yancy->filters ), @{ $res->{items} }
+    ] if $c->stash( 'filters_out' );
 
     return $c->render(
         status => 200,
@@ -93,7 +101,12 @@ sub list_items {
 #pod stash key C<collection>.
 #pod
 #pod The new item is extracted from the OpenAPI input, under parameter name
-#pod C<newItem>, and must be a hash/JSON "object".
+#pod C<newItem>, and must be a hash/JSON "object". It will be filtered by
+#pod filters conforming with L<Mojolicious::Plugin::Yancy/yancy.filter.add>
+#pod that are passed in the array-ref in stash key C<filters>, after the
+#pod collection and property filters have been applied.
+#pod
+#pod The return value is filtered like each result is in L</list_items>.
 #pod
 #pod =cut
 
@@ -102,9 +115,14 @@ sub add_item {
     return unless $c->openapi->valid_input;
     my $coll = $c->stash( 'collection' );
     my $item = $c->yancy->filter->apply( $coll, $c->validation->param( 'newItem' ) );
+    $item = _apply_op_filters( $coll, $item, $c->stash( 'filters' ), $c->yancy->filters )
+        if $c->stash( 'filters' );
+    my $res = $c->yancy->backend->create( $coll, $item );
+    $res = _apply_op_filters( $coll, $res, $c->stash( 'filters_out' ), $c->yancy->filters )
+        if $c->stash( 'filters_out' );
     return $c->render(
         status => 201,
-        openapi => $c->yancy->backend->create( $coll, $item ),
+        openapi => $res,
     );
 }
 
@@ -116,6 +134,8 @@ sub add_item {
 #pod The item's ID field-name is in the stash key C<id_field>. The ID itself
 #pod is extracted from the OpenAPI input, under a parameter of that name.
 #pod
+#pod The return value is filtered like each result is in L</list_items>.
+#pod
 #pod =cut
 
 sub get_item {
@@ -123,9 +143,13 @@ sub get_item {
     return unless $c->openapi->valid_input;
     my $args = $c->validation->output;
     my $id = $args->{ $c->stash( 'id_field' ) };
+    my $coll = $c->stash( 'collection' );
+    my $res = _delete_null_values( $c->yancy->backend->get( $coll, $id ) );
+    $res = _apply_op_filters( $coll, $res, $c->stash( 'filters_out' ), $c->yancy->filters )
+        if $c->stash( 'filters_out' );
     return $c->render(
         status => 200,
-        openapi => _delete_null_values( $c->yancy->backend->get( $c->stash( 'collection' ), $id ) ),
+        openapi => $res,
     );
 }
 
@@ -137,6 +161,8 @@ sub get_item {
 #pod The item to be updated is determined as with L</get_item>, and what to
 #pod update it with is determined as with L</add_item>.
 #pod
+#pod The return value is filtered like each result is in L</list_items>.
+#pod
 #pod =cut
 
 sub set_item {
@@ -146,14 +172,19 @@ sub set_item {
     my $id = $args->{ $c->stash( 'id_field' ) };
     my $coll = $c->stash( 'collection' );
     my $item = $c->yancy->filter->apply( $coll, $args->{ newItem } );
+    $item = _apply_op_filters( $coll, $item, $c->stash( 'filters' ), $c->yancy->filters )
+        if $c->stash( 'filters' );
     $c->yancy->backend->set( $coll, $id, $item );
 
     # ID field may have changed
     $id = $item->{ $c->stash( 'id_field' ) } || $id;
 
+    my $res = _delete_null_values( $c->yancy->backend->get( $coll, $id ) );
+    $res = _apply_op_filters( $coll, $res, $c->stash( 'filters_out' ), $c->yancy->filters )
+        if $c->stash( 'filters_out' );
     return $c->render(
         status => 200,
-        openapi => _delete_null_values( $c->yancy->backend->get( $coll, $id ) ),
+        openapi => $res,
     );
 }
 
@@ -186,11 +217,25 @@ sub delete_item {
 # confusing to understand what the problem is
 sub _delete_null_values {
     for my $item ( @_ ) {
-        for my $key ( grep { !defined $item->{ $_ } } keys %$item ) {
-            delete $item->{ $key };
-        }
+        delete $item->{ $_ } for grep !defined $item->{ $_ }, keys %$item;
     }
-    return @_;
+    return wantarray ? @_ : $_[0];
+}
+
+#=sub _apply_op_filters
+#
+# Similar to the helper 'yancy.filter.apply' - filters input item,
+# returns updated version.
+sub _apply_op_filters {
+    my ( $coll, $item, $filters, $app_filters ) = @_;
+    $item = { %$item }; # no mutate input
+    for my $filter ( @$filters ) {
+        ( $filter, my @params ) = @$filter if ref $filter eq 'ARRAY';
+        my $sub = $app_filters->{ $filter };
+        die "Unknown filter: $filter" unless $sub;
+        $item = $sub->( $coll, $item, {}, @params );
+    }
+    $item;
 }
 
 1;
@@ -205,7 +250,7 @@ Yancy::Controller::Yancy::API - An OpenAPI REST controller for the Yancy editor
 
 =head1 VERSION
 
-version 1.022
+version 1.023
 
 =head1 DESCRIPTION
 
@@ -219,6 +264,10 @@ backend data. This API is used by the Yancy editor.
 List the items in a collection. The collection name should be in the
 stash key C<collection>.
 
+Each returned item will be filtered by filters conforming with
+L<Mojolicious::Plugin::Yancy/yancy.filter.add> that are passed in the
+array-ref in stash key C<filters_out>.
+
 C<$limit>, C<$offset>, and C<$order_by> may be provided as query parameters.
 
 =head2 add_item
@@ -227,7 +276,12 @@ Add a new item to the collection. The collection name should be in the
 stash key C<collection>.
 
 The new item is extracted from the OpenAPI input, under parameter name
-C<newItem>, and must be a hash/JSON "object".
+C<newItem>, and must be a hash/JSON "object". It will be filtered by
+filters conforming with L<Mojolicious::Plugin::Yancy/yancy.filter.add>
+that are passed in the array-ref in stash key C<filters>, after the
+collection and property filters have been applied.
+
+The return value is filtered like each result is in L</list_items>.
 
 =head2 get_item
 
@@ -237,6 +291,8 @@ stash key C<collection>.
 The item's ID field-name is in the stash key C<id_field>. The ID itself
 is extracted from the OpenAPI input, under a parameter of that name.
 
+The return value is filtered like each result is in L</list_items>.
+
 =head2 set_item
 
 Update an item in a collection. The collection should be in the stash
@@ -245,6 +301,8 @@ key C<collection>.
 The item to be updated is determined as with L</get_item>, and what to
 update it with is determined as with L</add_item>.
 
+The return value is filtered like each result is in L</list_items>.
+
 =head2 delete_item
 
 Delete an item from a collection. The collection name should be in the
