@@ -11,11 +11,14 @@ __PACKAGE__->add_columns(
     },
     user_id => {
         data_type => 'integer',
+        is_nullable => 1,
     },
-    qw/ title slug markdown html /,
+    title => { is_nullable => 1 },
+    slug => { is_nullable => 1 },
+    markdown => { is_nullable => 1 },
+    html => { is_nullable => 1 },
     is_published => {
         data_type => 'boolean',
-        is_nullable => 0,
         default_value => 0,
     },
 );
